/******************************************************************************
    ncube.c: NINTENDO64 N-Cube shape data

 *****************************************************************************/

#include <ultra64.h>

#define CUBE_RED                        219,16,28,255
#define CUBE_GREEN                      58,188,85,255
#define CUBE_BLUE                       82,75,152,255
#define CUBE_YELLOW                     255,196,0,255

#define gsDPSetMaterial(a)              gsDPPipeSync(), gsDPSetEnvColor(a)

#define G_CC_ENVSHADE                   SHADE,0,ENVIRONMENT,0,0,0,0,ENVIRONMENT


/*****************************************************************************/
/*                                                                           */
/*              N-Cube vertex data                                           */
/*                                                                           */
/*****************************************************************************/

static Vtx NCubeVertex[] = {
       {    196,    434,   -482,     0,   576, -1054,     0,    0, -127, 255}, 
       {   -195,   -433,   -482,     0,  1404,   990,     0,    0, -127, 255}, 
       {   -195,      0,   -482,     0,  1404,     0,     0,    0, -127, 255}, 
       {   -482,   -433,   -482,     0,  2012,   990,     0,    0, -127, 255}, 
       {   -482,    434,   -482,     0,  2012, -1054,     0,    0, -127, 255}, 
       {    196,      0,   -482,     0,   576,   -30,     0,    0, -127, 255}, 
       {   -195,    434,   -482,     0,  1404, -1054,     0,    0, -127, 255}, 
       {   -482,    434,    196,     0,     0, -1054,     0,    0, -127, 255}, 
       {   -195,      0,    196,     0,   576,     0,     0,    0, -127, 255}, 
       {   -482,      0,    196,     0,     0,     0,     0,    0, -127, 255}, 
       {   -195,    434,    196,     0,   576, -1054,     0,    0, -127, 255}, 
       {    196,      0,    196,     0,  1404,     0,     0,    0, -127, 255}, 
       {   -195,      0,    196,     0,   576,   -30,     0,    0, -127, 255}, 
       {    483,   -433,   -482,     0,     0,   990,     0,    0, -127, 255}, 
       {    483,    434,   -482,     0,     0, -1054,     0,    0, -127, 255}, 
       {    196,   -433,    196,     0,  1404,   990,     0,    0, -127, 255}, 
       {    196,      0,    196,     0,  1404,     0,     0,    0, -127, 255}, 
       {    483,      0,    196,     0,  2012,     0,     0,    0, -127, 255}, 
       {    483,   -433,    196,     0,  2012,   990,     0,    0, -127, 255}, 
       {    196,   -433,    196,     0,  1404,   990,     0,    0, -127, 255}, 
       {    483,   -433,   -482,     0,     0,   990,     0,    0, -127, 255}, 
       {    196,      0,   -482,     0,   576,   -30,     0,    0, -127, 255}, 
       {    196,    434,   -482,     0,   576, -1054,     0,    0, -127, 255}, 
       {    483,   -433,   -482,     0,   -30,   990,     0,    0, -127, 255}, 
       {    196,   -433,   -482,     0,   576,   990,     0,    0, -127, 255}, 
       {    483,    434,   -482,     0,     0,     0,     0,  127,    0, 255}, 
       {    196,    434,   -195,     0,   644,   576,     0,  127,    0, 255}, 
       {    483,    434,   -195,     0,   644,     0,     0,  127,    0, 255}, 
       {   -482,    434,   -195,     0,     0,   644,     0,  127,    0, 255}, 
       {   -195,    434,   -195,     0,   576,   644,     0,  127,    0, 255}, 
       {   -195,    434,   -482,     0,   576,     0,     0,  127,    0, 255}, 
       {   -482,    434,   -482,     0,     0,     0,     0,  127,    0, 255}, 
       {   -195,    434,    196,     0,   644,   576,     0,  127,    0, 255}, 
       {   -482,    434,    196,     0,   644,     0,     0,  127,    0, 255}, 
       {   -482,    434,    483,     0,     0,     0,     0,  127,    0, 255}, 
       {   -195,    434,    483,     0,     0,   576,     0,  127,    0, 255}, 
       {    483,    434,    196,     0,     0,   644,     0,  127,    0, 255}, 
       {    196,    434,    483,     0,   576,     0,     0,  127,    0, 255}, 
       {    483,    434,    483,     0,     0,     0,     0,  127,    0, 255}, 
       {    196,    434,    196,     0,   576,   644,     0,  127,    0, 255}, 
       {    196,    434,   -482,     0,     0,   576,     0,  127,    0, 255}, 
       {   -195,    434,    483,     0,  1404, -1054,    94,   85,    0, 255}, 
       {    196,      0,    483,     0,  1404,     0,    94,   85,    0, 255}, 
       {   -195,    434,    196,     0,  2012, -1054,    94,   85,    0, 255}, 
       {    196,      0,    196,     0,  2012,     0,    94,   85,    0, 255}, 
       {   -482,   -433,    483,     0,  2012,   990,  -127,    0,    0, 255}, 
       {   -482,    434,    483,     0,  2012, -1054,  -127,    0,    0, 255}, 
       {   -482,    434,    196,     0,  1404, -1054,  -127,    0,    0, 255}, 
       {   -482,      0,    196,     0,  1404,     0,  -127,    0,    0, 255}, 
       {   -482,   -433,    196,     0,  1404,   990,  -127,    0,    0, 255}, 
       {   -482,      0,   -195,     0,   576,     0,  -127,    0,    0, 255}, 
       {   -482,    434,   -195,     0,   576, -1054,  -127,    0,    0, 255}, 
       {    196,   -433,   -482,     0,  2012,   990,  -127,    0,    0, 255}, 
       {    196,      0,   -195,     0,  1404,     0,  -127,    0,    0, 255}, 
       {    196,      0,   -482,     0,  2012,     0,  -127,    0,    0, 255}, 
       {    196,   -433,   -195,     0,  1404,   990,  -127,    0,    0, 255}, 
       {    196,    434,    483,     0,     0, -1054,  -127,    0,    0, 255}, 
       {    196,    434,    196,     0,   576, -1054,  -127,    0,    0, 255}, 
       {    196,      0,    196,     0,   576,     0,  -127,    0,    0, 255}, 
       {    196,      0,    483,     0,     0,     0,  -127,    0,    0, 255}, 
       {   -482,    434,   -482,     0,     0, -1054,  -127,    0,    0, 255}, 
       {   -482,   -433,   -195,     0,   576,   990,  -127,    0,    0, 255}, 
       {   -482,      0,   -195,     0,   576,     0,  -127,    0,    0, 255}, 
       {   -482,    434,   -482,     0,     0, -1054,  -127,    0,    0, 255}, 
       {   -482,   -433,   -482,     0,     0,   990,  -127,    0,    0, 255}, 
       {    483,   -433,   -195,     0,  2012,   990,     0,  -85,   94, 255}, 
       {    196,      0,    196,     0,  1404,     0,     0,  -85,   94, 255}, 
       {    196,   -433,   -195,     0,  1404,   990,     0,  -85,   94, 255}, 
       {    483,      0,    196,     0,  2012,     0,     0,  -85,   94, 255}, 
       {    196,    434,   -482,     0,  1404, -1054,   -94,   85,    0, 255}, 
       {   -195,      0,   -482,     0,  1404,     0,   -94,   85,    0, 255}, 
       {    196,    434,   -195,     0,  2012, -1054,   -94,   85,    0, 255}, 
       {   -195,      0,   -195,     0,  2012,     0,   -94,   85,    0, 255}, 
       {   -482,    434,   -195,     0,     0, -1054,     0,   85,   94, 255}, 
       {   -482,      0,    196,     0,     0,     0,     0,   85,   94, 255}, 
       {   -195,      0,    196,     0,   576,     0,     0,   85,   94, 255}, 
       {   -195,    434,   -195,     0,   576, -1054,     0,   85,   94, 255}, 
       {   -482,   -433,    483,     0,   382,   314,     0, -127,    0, 255}, 
       {   -482,   -433,    196,     0,   382,   990,     0, -127,    0, 255}, 
       {   -195,   -433,    196,     0,   990,   990,     0, -127,    0, 255}, 
       {   -482,   -433,   -482,     0,     0,   382,     0, -127,    0, 255}, 
       {   -195,   -433,   -482,     0,     0,   990,     0, -127,    0, 255}, 
       {   -482,   -433,   -195,     0,   644,   382,     0, -127,    0, 255}, 
       {   -195,   -433,   -195,     0,   644,   990,     0, -127,    0, 255}, 
       {    196,   -433,   -195,     0,   990,   990,     0, -127,    0, 255}, 
       {    196,   -433,   -482,     0,   990,   314,     0, -127,    0, 255}, 
       {    483,   -433,   -482,     0,   382,   314,     0, -127,    0, 255}, 
       {    483,   -433,   -195,     0,   382,   990,     0, -127,    0, 255}, 
       {    196,   -433,    483,     0,     0,   990,     0, -127,    0, 255}, 
       {    196,   -433,    196,     0,   644,   990,     0, -127,    0, 255}, 
       {    483,   -433,    196,     0,   644,   382,     0, -127,    0, 255}, 
       {    483,   -433,    483,     0,     0,   382,     0, -127,    0, 255}, 
       {   -195,   -433,    483,     0,   990,   314,     0, -127,    0, 255}, 
       {   -195,   -433,   -482,     0,  2012,   990,    94,  -85,    0, 255}, 
       {    196,      0,   -195,     0,  1404,     0,    94,  -85,    0, 255}, 
       {   -195,   -433,   -195,     0,  1404,   990,    94,  -85,    0, 255}, 
       {    196,      0,   -482,     0,  2012,     0,    94,  -85,    0, 255}, 
       {    196,      0,    483,     0,  1404,     0,     0,    0,  127, 255}, 
       {    196,   -433,    483,     0,  1404,   990,     0,    0,  127, 255}, 
       {    483,    434,    483,     0,  2010, -1050,     0,    0,  127, 255}, 
       {    483,    434,    483,     0,  2012, -1054,     0,    0,  127, 255}, 
       {    196,    434,    483,     0,  1404, -1054,     0,    0,  127, 255}, 
       {   -195,    434,    483,     0,   576, -1054,     0,    0,  127, 255}, 
       {   -195,      0,    483,     0,   576,   -30,     0,    0,  127, 255}, 
       {    483,   -433,    483,     0,  2012,   990,     0,    0,  127, 255}, 
       {    483,    434,   -195,     0,  2012, -1054,     0,    0,  127, 255}, 
       {    196,    434,   -195,     0,  1404, -1054,     0,    0,  127, 255}, 
       {    196,      0,   -195,     0,  1404,     0,     0,    0,  127, 255}, 
       {    483,      0,   -195,     0,  2012,     0,     0,    0,  127, 255}, 
       {   -482,   -433,    483,     0,     0,   990,     0,    0,  127, 255}, 
       {   -195,      0,   -195,     0,   576,     0,     0,    0,  127, 255}, 
       {    196,      0,   -195,     0,  1404,   -30,     0,    0,  127, 255}, 
       {   -195,   -433,   -195,     0,   576,   990,     0,    0,  127, 255}, 
       {   -195,      0,   -195,     0,   576,     0,     0,    0,  127, 255}, 
       {   -482,      0,   -195,     0,     0,     0,     0,    0,  127, 255}, 
       {   -482,   -433,   -195,     0,     0,   990,     0,    0,  127, 255}, 
       {   -195,   -433,   -195,     0,   576,   990,     0,    0,  127, 255}, 
       {   -482,   -433,    483,     0,     0,   990,     0,    0,  127, 255}, 
       {   -195,   -433,    483,     0,   576,   990,     0,    0,  127, 255}, 
       {   -195,      0,    483,     0,   576,   -30,     0,    0,  127, 255}, 
       {   -195,    434,    483,     0,   576, -1054,     0,    0,  127, 255}, 
       {   -482,    434,    483,     0,     0, -1054,     0,    0,  127, 255}, 
       {   -482,   -433,    196,     0,     0, -1054,     0,  -85,  -94, 255}, 
       {   -482,      0,   -195,     0,     0,     0,     0,  -85,  -94, 255}, 
       {   -195,      0,   -195,     0,   576,     0,     0,  -85,  -94, 255}, 
       {   -195,   -433,    196,     0,   576, -1054,     0,  -85,  -94, 255}, 
       {   -195,      0,    196,     0,   576,     0,   127,    0,    0, 255}, 
       {   -195,      0,   -195,     0,  1404,     0,   127,    0,    0, 255}, 
       {   -195,    434,   -195,     0,  1404, -1054,   127,    0,    0, 255}, 
       {    483,   -433,   -482,     0,  2012,   990,   127,    0,    0, 255}, 
       {    483,      0,   -195,     0,  1404,     0,   127,    0,    0, 255}, 
       {    483,   -433,   -195,     0,  1404,   990,   127,    0,    0, 255}, 
       {    483,    434,   -482,     0,  2012, -1054,   127,    0,    0, 255}, 
       {    483,    434,   -195,     0,  1404, -1054,   127,    0,    0, 255}, 
       {    483,      0,    196,     0,   576,     0,   127,    0,    0, 255}, 
       {    483,    434,    196,     0,   576, -1054,   127,    0,    0, 255}, 
       {   -195,    434,   -482,     0,  2012, -1054,   127,    0,    0, 255}, 
       {   -195,      0,   -482,     0,  2012,     0,   127,    0,    0, 255}, 
       {   -195,   -433,    196,     0,   576,   990,   127,    0,    0, 255}, 
       {    483,   -433,    483,     0,     0,   990,   127,    0,    0, 255}, 
       {    483,   -433,    196,     0,   576,   990,   127,    0,    0, 255}, 
       {   -195,   -433,    483,     0,     0,   990,   127,    0,    0, 255}, 
       {   -195,      0,    196,     0,   576,     0,   127,    0,    0, 255}, 
       {   -195,      0,    483,     0,     0,     0,   127,    0,    0, 255}, 
       {   -195,   -433,    196,     0,   576,   990,   127,    0,    0, 255}, 
       {    483,    434,    196,     0,   576, -1054,   127,    0,    0, 255}, 
       {    483,    434,    483,     0,     0, -1054,   127,    0,    0, 255}, 
       {    483,   -433,    483,     0,     0,   990,   127,    0,    0, 255}, 
       {    483,      0,    196,     0,   576,     0,   127,    0,    0, 255}, 
       {    196,   -433,    483,     0,     0,     0,   -94,  -85,    0, 255}, 
       {   -195,      0,    196,     0,   576,   990,   -94,  -85,    0, 255}, 
       {    196,   -433,    196,     0,   576,     0,   -94,  -85,    0, 255}, 
       {   -195,      0,    483,     0,     0,   990,   -94,  -85,    0, 255}, 
       {    483,    434,    196,     0,  1404, -1054,     0,   85,  -94, 255}, 
       {    483,      0,   -195,     0,  1404,     0,     0,   85,  -94, 255}, 
       {    196,    434,    196,     0,  2012, -1054,     0,   85,  -94, 255}, 
       {    196,      0,   -195,     0,  2012,     0,     0,   85,  -94, 255}, 
};

/*****************************************************************************/
/*                                                                           */
/*              N-Cube display list                                          */
/*                                                                           */
/*****************************************************************************/

Mtx NCube_mtx[] = {
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000001,
  0x20000000, 0x00000000, 0x00002000, 0x00000000,
  0x00000000, 0x20000000, 0x00000000, 0x00000000,
};

Gfx n64_model0[] = {
        gsDPPipeSync(),
        gsDPSetCycleType(G_CYC_1CYCLE),
        gsSPClearGeometryMode(G_SHADING_SMOOTH),
        gsDPSetCombineMode(G_CC_ENVSHADE, G_CC_ENVSHADE),
        gsDPSetRenderMode(G_RM_AA_ZB_OPA_SURF, G_RM_AA_ZB_OPA_SURF2),
#if 1
	gsSPMatrix(&NCube_mtx, G_MTX_MODELVIEW | G_MTX_MUL | G_MTX_NOPUSH),
#endif
        gsDPSetMaterial(CUBE_GREEN),
        gsSPVertex(&NCubeVertex[0], 16, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 1, 3, 4, 0),
        gsSP2Triangles( 1, 4, 2, 0, 0, 5, 1, 0),
        gsSP2Triangles( 4, 6, 2, 0, 7, 8, 9, 0),
        gsSP2Triangles( 7,10, 8, 0,11,12,10, 0),
        gsSP2Triangles(13, 0,14, 0,11,15,12, 0),
        gsSPVertex(&NCubeVertex[16],  9, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 2, 3, 0),
        gsSP2Triangles( 4, 5, 6, 0, 7, 8, 5, 0),

        gsSPVertex(&NCubeVertex[97], 16, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 3, 4, 0, 0),
        gsSP2Triangles( 5, 1, 0, 0, 5, 6, 1, 0),
        gsSP2Triangles( 3, 1, 7, 0, 8, 9,10, 0),
        gsSP2Triangles( 8,10,11, 0, 5,12, 6, 0),
        gsSP2Triangles(13,14, 9, 0,13,15,14, 0),
        gsSPVertex(&NCubeVertex[113],  9, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 2, 3, 0),
        gsSP2Triangles( 4, 5, 6, 0, 7, 8, 4, 0),

        gsSPVertex(&NCubeVertex[153],  4, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 1, 3, 2, 0),

        gsSPVertex(&NCubeVertex[73],  4, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 2, 3, 0),


        gsDPSetMaterial(CUBE_BLUE),
        gsSPVertex(&NCubeVertex[45], 16, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 3, 4, 0),
        gsSP2Triangles( 0, 2, 3, 0, 5, 4, 3, 0),
        gsSP2Triangles( 6, 5, 3, 0, 7, 8, 9, 0),
        gsSP2Triangles( 7,10, 8, 0,11,12,13, 0),
        gsSP2Triangles(11,13,14, 0, 8,13,12, 0),
        gsSP2Triangles( 8,10,13, 0, 5, 6,15, 0),
        gsSPVertex(&NCubeVertex[61],  4, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 2, 3, 0),

        gsSPVertex(&NCubeVertex[126], 15, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 3, 4, 5, 0),
        gsSP2Triangles( 6, 7, 3, 0, 3, 7, 4, 0),
        gsSP2Triangles( 8, 5, 4, 0, 9, 8, 4, 0),
        gsSP2Triangles(10, 2, 1, 0,10, 1,11, 0),
        gsSP2Triangles( 0,12, 1, 0,13,14, 8, 0),
        gsSPVertex(&NCubeVertex[141],  8, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 3, 1, 0),
        gsSP2Triangles( 4, 5, 6, 0, 4, 6, 7, 0),

        gsSPVertex(&NCubeVertex[149],  4, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 3, 1, 0),

        gsSPVertex(&NCubeVertex[93],  4, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 3, 1, 0),


        gsDPSetMaterial(CUBE_RED),
        gsSPVertex(&NCubeVertex[69],  4, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 1, 3, 2, 0),

        gsSPVertex(&NCubeVertex[41],  4, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 1, 3, 2, 0),

        gsSPVertex(&NCubeVertex[122],  4, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 2, 3, 0),

        gsSPVertex(&NCubeVertex[65],  4, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 0, 3, 1, 0),


        gsDPSetMaterial(CUBE_YELLOW),
        gsSPVertex(&NCubeVertex[25], 16, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 3, 4, 5, 0),
        gsSP2Triangles( 3, 5, 6, 0, 7, 8, 9, 0),
        gsSP2Triangles( 7, 9,10, 0,11,12,13, 0),
        gsSP2Triangles(11,14,12, 0, 0,15, 1, 0),

        gsSPVertex(&NCubeVertex[77], 16, 0), 
        gsSP2Triangles( 0, 1, 2, 0, 3, 4, 5, 0),
        gsSP2Triangles( 4, 6, 5, 0, 7, 8, 9, 0),
        gsSP2Triangles(10, 7, 9, 0,11,12,13, 0),
        gsSP2Triangles(14,11,13, 0,15, 0, 2, 0),
        gsSPEndDisplayList() 
};

